@@ -13,13 +13,9 @@ import isort
 import patoolib
 from colorama import Fore
 from portmod._cli.pybuild import create_manifest
-from portmod.atom import Atom, version_gt
-from portmod.colour import colour
 from portmod.download import download, get_download
-from portmod.fs.util import get_hash
 from portmod.globals import env
 from portmod.loader import load_file, load_pkg
-from portmod.masters import get_masters
 from portmod.merge import configure
 from portmod.prompt import prompt_bool
 from portmod.pybuild import File, InstallDir, parse_arrow
@@ -27,6 +23,10 @@ from portmod.repo import Repo, get_repo
 from portmod.repo.metadata import get_categories
 from portmod.source import Source
 from portmod.util import get_newest
+from portmodlib.atom import Atom, version_gt
+from portmodlib.colour import colour
+from portmodlib.fs import get_hash
+from portmodlib.masters import get_masters
 from redbaron import AtomtrailersNode, CallNode, RedBaron
 
 from .atom import parse_atom
@@ -151,7 +151,7 @@ def generate_build_files(
     author = mod.get("author")
     needs_cleaning = mod.get("needs_cleaning")
 
-    sources = []
+    sources: List[Source] = []
     source_string = url
 
     parsed = urllib.parse.urlparse(url)
@@ -215,17 +215,17 @@ def generate_build_files(
             url = None
             file = None
             downloaded = True
-            sources = nexus_data.files
-            source_string = " ".join(sources)
+            sources = [Source(file, file) for file in nexus_data.files]
+            source_string = " ".join([source.name for source in sources])
             if nexus_data.nudity:
                 REQUIRED_USE.append("nudity")
             author = author or nexus_data.author
             OTHER_FIELDS["NEXUS_URL"] = f'"{nexus_data.homepage}"'
             CLASS.append("NexusMod")
-            if "pyclass" not in OTHER_IMPORTS:
-                OTHER_IMPORTS["pyclass"] = ["NexusMod"]
-            elif "NexusMod" not in OTHER_IMPORTS["pyclass"]:
-                OTHER_IMPORTS["pyclass"].append("NexusMod")
+            if "common.nexus" not in OTHER_IMPORTS:
+                OTHER_IMPORTS["common.nexus"] = ["NexusMod"]
+            elif "NexusMod" not in OTHER_IMPORTS["common.nexus"]:
+                OTHER_IMPORTS["common.nexus"].append("NexusMod")
 
     elif parsed.hostname == "mw.modhistory.com":
         homepage = url
@@ -237,10 +237,10 @@ def generate_build_files(
     elif parsed.hostname == "github.com" or parsed.hostname == "gitlab.com":
         source_string = ""
         CLASS.insert(0, "Git")
-        if "pyclass" not in OTHER_IMPORTS:
-            OTHER_IMPORTS["pyclass"] = ["Git"]
-        elif "Git" not in OTHER_IMPORTS["pyclass"]:
-            OTHER_IMPORTS["pyclass"].append("Git")
+        if "common.git" not in OTHER_IMPORTS:
+            OTHER_IMPORTS["common.git"] = ["Git"]
+        elif "Git" not in OTHER_IMPORTS["common.git"]:
+            OTHER_IMPORTS["common.git"].append("Git")
         OTHER_FIELDS["GIT_SRC_URI"] = '"{url}"'
 
         name, _ = os.path.splitext(os.path.basename(url))
@@ -330,26 +330,26 @@ def generate_build_files(
 
     for source in sources:
         # Search for data directories
-        outdir = os.path.join(env.TMP_DIR, source)
+        outdir = os.path.join(env.TMP_DIR, source.name)
         dirs = find_data_dirs(outdir)
-        data_dirs.append((source, dirs))
+        data_dirs.append((source.name, dirs))
         print(
             "Detected the following data directories for {}: {}".format(
-                source, [dir.PATH for dir in dirs]
+                source.name, [dir.PATH for dir in dirs]
             )
         )
 
         for directory in dirs:
             (esps, bsas) = find_esp_bsa(os.path.join(outdir, directory.PATH))
             if bsas:
-                directory.add_kwarg("ARCHIVES", [File(bsa) for bsa in bsas])
+                directory._add_kwarg("ARCHIVES", [File(bsa) for bsa in bsas])
 
-            source_name, _ = os.path.splitext(source)
+            source_name, _ = os.path.splitext(source.name)
             if source_name.endswith(".tar"):
                 source_name, _ = os.path.splitext(source_name)
 
             texture_size = get_dominant_texture_size(
-                os.path.join(env.TMP_DIR, source, directory.PATH)
+                os.path.join(env.TMP_DIR, source.name, directory.PATH)
             )
 
             if texture_size:
@@ -360,6 +360,8 @@ def generate_build_files(
             for esp in esps:
                 esp_path = os.path.join(outdir, directory.PATH, esp)
                 print("Masters of esp {} are {}".format(esp, get_masters(esp_path)))
+                dep_atom = None
+                dep_use = None
                 try:
                     (dep_atom, dep_use) = get_esp_deps(
                         esp_path,
@@ -383,7 +385,9 @@ def generate_build_files(
                 CLEAN = False
                 TR_PATCH = False
 
-                if needs_cleaning:
+                if needs_cleaning and dep_atom:
+                    # FIXME: This won't work without a prefix
+                    # Ideally, a temporary prefix could be set up
                     configure(
                         dep_atom,
                         no_confirm=True,
@@ -396,18 +400,18 @@ def generate_build_files(
                         CLEAN = True
                         if "CleanPlugin" not in CLASS:
                             CLASS.insert(0, "CleanPlugin")
-                        if "pyclass" not in OTHER_IMPORTS:
-                            OTHER_IMPORTS["pyclass"] = ["CleanPlugin"]
-                        elif "CleanPlugin" not in OTHER_IMPORTS["pyclass"]:
-                            OTHER_IMPORTS["pyclass"].append("CleanPlugin")
-                        build_deps |= dep_atom
+                        if "common.util" not in OTHER_IMPORTS:
+                            OTHER_IMPORTS["common.util"] = ["CleanPlugin"]
+                        elif "CleanPlugin" not in OTHER_IMPORTS["common.util"]:
+                            OTHER_IMPORTS["comon.util"].append("CleanPlugin")
+                        build_deps.add(dep_atom)
 
                 if "TR_Data.esm" in get_masters(esp_path):
                     TR_PATCH = True
-                    if "pyclass" not in OTHER_IMPORTS:
-                        OTHER_IMPORTS["pyclass"] = ["TRPatcher"]
-                    elif "TRPatcher" not in OTHER_IMPORTS["pyclass"]:
-                        OTHER_IMPORTS["pyclass"].append("TRPatcher")
+                    if "common.util" not in OTHER_IMPORTS:
+                        OTHER_IMPORTS["common.util"] = ["TRPatcher"]
+                    elif "TRPatcher" not in OTHER_IMPORTS["common.util"]:
+                        OTHER_IMPORTS["common.util"].append("TRPatcher")
                     if "TRPatcher" not in CLASS:
                         CLASS.insert(0, "TRPatcher")
                     print("TR Patching file {}".format(esp))
@@ -420,11 +424,14 @@ def generate_build_files(
                 if TR_PATCH:
                     plugin._add_kwarg("TR_PATCH", True)
 
-                plugin.comment = "# Deps: " + " ".join(sorted(dep_atom | dep_use)) + ""
+                if dep_atom is not None and dep_use is not None:
+                    plugin.comment = (
+                        "# Deps: " + " ".join(sorted(dep_atom | dep_use)) + ""
+                    )
                 PLUGINS.append(plugin)
 
             if PLUGINS:
-                directory.add_kwarg("PLUGINS", PLUGINS)
+                directory._add_kwarg("PLUGINS", PLUGINS)
 
             if texture_size:
                 directory.comment = f"# Texture Size: {texture_size}"
@@ -435,7 +442,7 @@ def generate_build_files(
                 directory.comment += "\n# FIXME: New Directory. Please check"
 
             if len(sources) > 1:
-                directory.add_kwarg("S", source_name)
+                directory._add_kwarg("S", source_name)
 
             INSTALL_DIRS.append(directory)
 
@@ -447,7 +454,7 @@ def generate_build_files(
 
     for source in sources:
         # Clean up files
-        path = os.path.join(env.TMP_DIR, source)
+        path = os.path.join(env.TMP_DIR, source.name)
         print(f"Cleaning up {path}")
         shutil.rmtree(path)
 
@@ -472,7 +479,7 @@ def generate_build_files(
     main_imports = ["Pybuild1"]
     if INSTALL_DIRS:
         main_imports.append("InstallDir")
-    if any(d.get_files() for d in INSTALL_DIRS):
+    if any(list(d.get_files()) for d in INSTALL_DIRS):
         main_imports.append("File")
 
     if imports:
@@ -503,7 +510,7 @@ def generate_build_files(
             )
             index += 1
 
-    Mod = pybuild.find("class", "Mod")
+    Mod = pybuild.find("class", "Package")
 
     values = {
         "NAME": f'"{name}"',
@@ -545,7 +552,7 @@ def generate_build_files(
 
         # Update S if present
         if Mod.find("name", "S") and len(sources) == 1:
-            source_name, _ = os.path.splitext(sources[0])
+            source_name, _ = os.path.splitext(sources[0].name)
             if source_name.endswith(".tar"):
                 source_name, _ = os.path.splitext(source_name)
             Mod.find("name", "S").parent.value = f'"{source_name}"'
@@ -553,10 +560,11 @@ def generate_build_files(
         valuestr = "\n    ".join(
             [f"{key}={values[key]}" for key in sorted(values.keys())]
         )
-        pybuild.append(f'class Mod({", ".join(reversed(CLASS))}):\n   {valuestr}')
-        Mod = pybuild.find("class", "Mod")
+        pybuild.append(f'class Package({", ".join(reversed(CLASS))}):\n   {valuestr}')
+        Mod = pybuild.find("class", "Package")
         Mod.append(f'SRC_URI="{source_string}"')
 
+    INSTALL_DIRS = sorted(INSTALL_DIRS, key=lambda x: (x.S, x.PATH))
     if Mod.find("name", "INSTALL_DIRS"):
         dirs = [
             node
@@ -582,17 +590,10 @@ def generate_build_files(
                     idir = node.value[1]
                     path = idir.value[0]
                     S = idir.find("name", "S")
-                    SOURCE = idir.find("name", "SOURCE")
                     if S:
                         entirepath = os.path.join(
                             str(S.parent.value).strip('"'), str(path.value).strip('"')
                         )
-                    elif SOURCE:
-                        source_str = str(SOURCE.parent.value).strip('"')
-                        source_str, _ = os.path.splitext(source_str)
-                        entirepath = os.path.join(
-                            source_str, str(path.value).strip('"')
-                        )
                     else:
                         entirepath = str(path.value).strip('"')
 
@@ -639,7 +640,7 @@ def generate_build_files(
         os.makedirs(os.path.join(REPO.location, "profiles"), exist_ok=True)
         metadata_file = os.path.join(REPO.location, "profiles", "repo_name")
         with open(metadata_file, "w") as file:
-            print("user-repo", file=file)
+            print("local", file=file)
 
         layout_file = os.path.join(REPO.location, "metadata", "layout.conf")
         os.makedirs(os.path.dirname(layout_file))
@@ -650,7 +651,7 @@ def generate_build_files(
         # Write user import repo to repos.cfg
         with open(env.REPOS_FILE, "a") as file:
             userstring = """
-[user]
+[local]
 location = {}
 auto_sync = False
 masters = openmw
@@ -672,7 +673,7 @@ priority = 50
 
     print("Exporting pybuild to {}".format(filename))
     with open(filename, "w") as file:
-        print(build_file, file=file)
+        print(build_file, file=file, end="")
 
     if parsed.hostname == "github.com" or parsed.hostname == "gitlab.com":
         # Create Live Pybuild
@@ -700,4 +701,4 @@ priority = 50
 def create_metadata(file, author):
     with open(file, "w") as metadata:
         print("upstream:", file=metadata)
-        print(f"    maintainer: !person {author}", file=metadata)
+        print(f"    maintainer: {author}", file=metadata)
