@@ -6,6 +6,24 @@
 import requests
 from bs4 import BeautifulSoup
 
+from ..atom import parse_version
+from . import PackageSource
+
+
+class ModhistorySource(PackageSource):
+    def __init__(self, idnum: int):
+        self.id = idnum
+
+    def get_url(self) -> str:
+        return f"http://mw.modhistory.com/download--{self.id}"
+
+    def get_newest_version(self) -> str:
+        info = get_modhistory_info(self.get_url())
+        return parse_version(info["Version"])
+
+    def __hash__(self):
+        return hash(self.id)
+
 
 def get_modhistory_info(url):
     """
